/*****************************************************************************
 *
 *  kv5500 - utility library for kv5500 like phone.
 *
 * Copyright (c) 2006 Hyo geol, Lee
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 *   3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/
#include <cstring>

#include <fstream>

#include <time.h>

#include "kv5500.hpp"
#include "kv5500_file.hpp"

using namespace KV5500;

FileInfoScanner::~FileInfoScanner()
{}

bool FileInfoScanner::DoScan(struct usb_device* dev)
{
    assert(dev);

    try {

        UsbDev udev(dev);

        /// vendor 1004 and product 6000, and this is not correct test.
        if(DescCheck(dev)) {

            if(!WakeUp(udev)) {

                printf("Cannot wake up phone.\n");

                return false;
            }

            if(!Request(udev))
                return false;
        }

    } catch(std::exception& e) {

        printf("%s\n", e.what());
    }

    return true;
}

bool FileInfoScanner::Request(UsbDev& udev)
{
    unsigned int totalNum = 0;
    if(!GetTotalNum(udev, numReqTempl_, numReqSize_, totalNum) || totalNum > 256)
        return false;

    if(totalNum == 0)
        return true;

    std::string name;
    unsigned int fileSize = 0;
    for(unsigned int i = 0; i < totalNum; ++i) {

        nameReqTempl_[3] = i + 1;
        if(udev.BulkWrite(nameReqTempl_, nameReqSize_, 5000) < 0) {

            printf("Got error while send.\n");

            return false;
        }

        if(RecvName(udev, name) < 0)
            return false;

        sizeReqTempl_[3] = i + 1;
        if(udev.BulkWrite(sizeReqTempl_, sizeReqSize_, 5000) < 0) {

            printf("Got error while send.\n");

            return false;
        }

        if(RecvSize(udev, fileSize) < 0)
            return false;

        Add(FileInfo(name, fileSize));
    }

    return true;
}

void KV5500::PrintFileInfoVec(const std::vector<FileInfo>& v)
{
    if(v.empty()) {

        printf("Nothing.\n");

        return;
    }

    const unsigned int total = v.size();
    const std::vector<FileInfo>::const_iterator end = v.end();
    unsigned int count = 1;
    for(std::vector<FileInfo>::const_iterator i = v.begin();
        i != end;
        ++i, ++count) {

        printf("-------------------------------------------%03d/%03d---\n", count, total);
        PrintFileInfo(*i);
    }
}

const char* FileGetNum::what() const throw()
{
    return "Request file number error.";
}

GetFileScanner::~GetFileScanner()
{}

bool GetFileScanner::DoScan(struct usb_device* dev)
{
    assert(dev);

    try {

        UsbDev udev(dev);

        /// vendor 1004 and product 6000, and this is not correct test.
        if(DescCheck(dev)) {

            if(!WakeUp(udev)) {

                printf("Cannot wake up phone.\n");

                return false;
            }

            if(!CheckFileNum(udev))
                return false;

            if(!PutFile(udev))
                return false;
        }

    } catch(std::exception& e) {

        printf("%s\n", e.what());
    }

    return true;
}

bool GetFileScanner::CheckFileNum(UsbDev& udev)
{
    unsigned int totalNum = 0;
    GetTotalNum(udev, numReqTempl_, numReqSize_, totalNum);
    if(totalNum > 256 || num_ > totalNum) {

        printf("Total %d files.\n", totalNum);

        return false;
    }

    return true;
}

bool GetFileScanner::SendPrepareTransfer(UsbDev& udev)
{
    if(udev.BulkWrite(prepareReqTempl_, prepareReqSize_, 5000) < 0) {

        printf("Got error while send.\n");

        return false;
    }

    char buf[1000];
    udev.BulkRead(buf, 1000, 5000);

    return true;
}

bool GetFileScanner::PutFile(UsbDev& udev)
{
    const std::string filename = GetFilename();

    sizeReqTempl_[3] = num_;
    if(udev.BulkWrite(sizeReqTempl_, sizeReqSize_, 5000) < 0) {

        printf("Got error while send.\n");

        return false;
    }

    char buf[3072];
    const int ret = udev.BulkRead(buf, sizeof(buf), 5000);

    /// assume ret must be over 11 bytes.
    if(ret <= 11) {

        printf("Got error while recv.\n");

        return false;
    }

    const unsigned int fileSize = GetRevEndian(buf + 8);

    int leftSize = fileSize;
    std::ofstream os(filename.c_str());
    if(!os) {

        printf("Cannot open file");

        return false;
    }

    SendPrepareTransfer(udev);

    printf("Writing %s .......\n", filename.c_str());

    unsigned int i = 0;
    transReqTempl_[3] = num_;
    while(leftSize > 0) {

        PrintTransferProcess(fileSize - leftSize, fileSize);

        transReqTempl_[7] = i;
        if(udev.BulkWrite(transReqTempl_, transReqSize_, 5000) < 0) {

            printf("\nGot error while send.\n");

            return false;
        }

        const int ret2 = udev.BulkRead(buf, sizeof(buf), 5000);
        /// assume ret must be over 15 bytes.
        if(ret2 <= 15)
            return false;

        const unsigned int prefix = GetPrefixPos(buf);
        const unsigned int postfix = GetPostfixPos(buf, ret2);

        for(unsigned int j = prefix; (j < ret2 - postfix) && (leftSize > 0); ++j, --leftSize) {
            if(static_cast<unsigned char>(buf[j]) == 0x7d &&
               static_cast<unsigned char>(buf[j+1]) == 0x5d) {

                os << static_cast<char>(0x7d);
                    
                ++j;

            } else if(static_cast<unsigned char>(buf[j]) == 0x7d && 
                      static_cast<unsigned char>(buf[j+1]) == 0x5e) {

                os << static_cast<char>(0x7e);

                ++j;

            } else
                os << buf[j];
        }

        if(leftSize > 0 && i == 256)
            i = 0;
        else
            ++i;
    }

    if(leftSize == 0)
        printf("\rDone. %d Bytes transferred.\n", fileSize);
    else
        printf("\rError. %d left.\n", leftSize);

    return true;
}

int KV5500::RecvName(UsbDev& udev, std::string& name)
{
    char buf[1000];
    const int ret = udev.BulkRead(buf, sizeof(buf), 5000);

    /// assume ret must be over 37 bytes.
    if(ret <= 37)
        return -1;

    name = GetString(buf, 26, 11, 26, 0x00);

    return ret;
}

int KV5500::RecvSize(UsbDev& udev, unsigned int& size)
{
    char buf[1000];
    const int ret = udev.BulkRead(buf, sizeof(buf), 5000);

    /// assume ret must be over 54 bytes.
    if(ret <= 11)
        return -1;

    size = GetRevEndian(buf + 8);

    return ret;
}

bool KV5500::GetTotalNum(UsbDev& udev, char* const req, size_t reqSize, unsigned int& num)
{
    assert(req && reqSize > 2);

    if(udev.BulkWrite(req, reqSize, 5000) < 0) {

        printf("Got error while send.\n");

        return false;
    }

    return RecvNum(udev, num);
}

bool KV5500::RecvNum(UsbDev& udev, unsigned int& num)
{
    char rbuf[1000];
    const int ret = udev.BulkRead(rbuf, sizeof(rbuf), 5000);
    if(ret < 8)
        return false;

    num = static_cast<unsigned int>(rbuf[5]);

    return true;
}

unsigned int KV5500::GetPrefixPos(const char* const buf)
{
    assert(buf);

    unsigned int prefix = 0;
    unsigned int prefixCount = 0;
    while(prefixCount != 12) {

        prefix += HasSubstStr(buf, prefix) ? 2 : 1;

        ++prefixCount;
    }

    return prefix;
}
