/*****************************************************************************
 *
 *  kv5500 - utility library for kv5500 like phone.
 *
 * Copyright (c) 2006 Hyo geol, Lee
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 *   3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/
#ifndef KV5500_FILE_H
#define KV5500_FILE_H

#include <string>
#include <vector>
#include <exception>

#include <cstdio>
#include <assert.h>

#include "scanner.hpp"

namespace KV5500 {

    class UsbDev;

    class FileInfo {
    public:
        FileInfo(const std::string& name, unsigned int size);

        const std::string& GetName() const;
        unsigned int GetSize() const;

        void SetName(const char* str);
        void SetSize(unsigned int size);

    private:
        std::string name_;
        unsigned int fileSize_;
    };

    void PrintFileInfo(const FileInfo& fi);

    class FileInfoScanner : public Scanner {
    public:
        FileInfoScanner();

        virtual ~FileInfoScanner() = 0;

        void GetInfo(std::vector<FileInfo>& vec) const;
        unsigned int GetInfoNum() const;

    protected:
        void Add(const FileInfo& item);

        char numReqTempl_[5];
        const size_t numReqSize_;
        
        char nameReqTempl_[6];
        const size_t nameReqSize_;

        char sizeReqTempl_[8];
        const size_t sizeReqSize_;

    private:
        std::vector<FileInfo> fileInfoContainer_;

        bool DoScan(struct usb_device* dev);

        bool Request(UsbDev& udev);

        FileInfoScanner(const FileInfoScanner&);
        FileInfoScanner& operator=(const FileInfoScanner&);
    };

    void PrintFileInfoVec(const std::vector<FileInfo>& v);

    class FileGetNum : public std::exception {
        const char* what() const throw();
    };

    class GetFileScanner : public Scanner {
    public:
        GetFileScanner(unsigned int num);

        virtual ~GetFileScanner();

    protected:
        unsigned int GetReqNum() const;

        char numReqTempl_[5];
        const size_t numReqSize_;

        char sizeReqTempl_[8];
        const size_t sizeReqSize_;

        char prepareReqTempl_[5];
        const size_t prepareReqSize_;

        char transReqTempl_[9];
        const size_t transReqSize_;

    private:
        const unsigned int num_;

        bool DoScan(struct usb_device* dev);

        bool CheckFileNum(UsbDev& udev);
        virtual std::string GetFilename() const = 0;

        bool SendPrepareTransfer(UsbDev& udev);

        bool PutFile(UsbDev& udev);

        GetFileScanner(const GetFileScanner&);
        GetFileScanner& operator=(const GetFileScanner&);
    };

    /// @brief Get 4bytes reverse endian.
    unsigned int GetRevEndian(char* buf);

    /// @brief Recv file name from udev to name.
    int RecvName(UsbDev& udev, std::string& name);

    /// @brief Recv file size from udev to size.
    int RecvSize(UsbDev& udev, unsigned int& size);

    bool GetTotalNum(UsbDev& udev, char* const req, size_t reqSize, unsigned int& num);

    /// @brief Recv file number from udev to size.
    bool RecvNum(UsbDev& udev, unsigned int& num);

    void PrintTransferProcess(unsigned int complete, unsigned int total);

    /// @brief Check buf and buf + 1 is 7d 5e or 7d 5d.
    bool HasSubstStr(const char* const buf, const unsigned int pos);

    /// @brief Count substituted 12 bytes from begin.
    unsigned int GetPrefixPos(const char* const buf);
    /// @brief Count substituted 3 bytes from end.
    unsigned int GetPostfixPos(const char* const buf, const unsigned int bufSize);
}

/// inline functions.
using namespace KV5500;

inline FileInfo::FileInfo(const std::string& name, unsigned int size)
    : name_(name),
      fileSize_(size)
{}

inline const std::string& FileInfo::GetName() const
{
    return name_;
}

inline unsigned int FileInfo::GetSize() const
{
    return fileSize_;
}

inline void FileInfo::SetName(const char* str)
{
    assert(str);

    name_ = str;
}

inline void FileInfo::SetSize(unsigned int size)
{
    fileSize_ = size;
}

inline void KV5500::PrintFileInfo(const FileInfo& fi)
{
    printf(" Name  : %s\n", fi.GetName().c_str());
    printf(" Size  : %.2f MBytes\n", fi.GetSize() / 1024.0 / 1024.0);
}

inline FileInfoScanner::FileInfoScanner()
    : numReqSize_(5),
      nameReqSize_(6),
      sizeReqSize_(8)
{}

inline void FileInfoScanner::GetInfo(std::vector<FileInfo>& vec) const
{
    vec = fileInfoContainer_;
}

inline unsigned int FileInfoScanner::GetInfoNum() const
{
    return fileInfoContainer_.size();
}

inline void FileInfoScanner::Add(const FileInfo& item)
{
    fileInfoContainer_.push_back(item);
}

inline GetFileScanner::GetFileScanner(unsigned int num)
    : numReqSize_(5),
      sizeReqSize_(8),
      prepareReqSize_(5),
      transReqSize_(9),
      num_(num)
{
    if(num == 0 || num > 256)
        throw FileGetNum();
}

inline unsigned int GetFileScanner::GetReqNum() const
{
    return num_;
}

inline unsigned int KV5500::GetRevEndian(char* buf)
{
    assert(buf);

    return static_cast<unsigned char>(buf[0]) + 
        (static_cast<unsigned char>(buf[1]) << 8) + 
        (static_cast<unsigned char>(buf[2]) << 16) +
        (static_cast<unsigned char>(buf[3]) << 24);
}

inline void KV5500::PrintTransferProcess(unsigned int complete, unsigned int total)
{
    assert(complete <= total);

    printf("\r%d / %d (%.2f%%)", complete, total, 100.0 * complete / total);
}

inline bool KV5500::HasSubstStr(const char* const buf, const unsigned int pos)
{
    assert(buf);

    return static_cast<unsigned char>(buf[pos]) == 0x7d &&
        (static_cast<unsigned char>(buf[pos + 1]) == 0x5e ||
         static_cast<unsigned char>(buf[pos + 1]) == 0x5d);
}

inline unsigned int KV5500::GetPostfixPos(const char* const buf, const unsigned int bufSize)
{
    assert(buf && bufSize > 4);

    return HasSubstStr(buf, bufSize - 4) || HasSubstStr(buf, bufSize -3) ? 4 : 3;
}

#endif
