/*****************************************************************************
 *
 *  kv5500_phonebook - read a phonebook from kv5500 like phone.
 *
 * Copyright (c) 2006 Hyo geol, Lee
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 *   3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/
#ifndef KV5500_PHONEBOOK_H
#define KV5500_PHONEBOOK_H

#include <vector>
#include <string>

#include "scanner.hpp"

/// If you send prefix 0xfe and postfix 0x7e, phone send some data.
///
/// fe 46 7e            First phone num of phonebook (size 215?)
///                     Name begin offset 9, name is KS encoding.
///                     Total telephone number begin offset + 33,
///                     each telephone number size maybe 33.
///
/// fe 47 7e            Second phone num of phonebook (size 215?)
///                     Name begin offset 11, that first phone num offset + 2.
///                     When you send 'fe 47 7e' again, you will get next num.
///                     I don't know total number of phone book. 
///                     I assume if third byte of data is 0x01(buf[2] == 0x01)
///                     it will be empty data.
///
/// Need root permission to access usb on FreeBSD, maybe also in GNU/Linux. :(
namespace KV5500 {

    class UsbDev;

    class PhonebookData {
    public:
        PhonebookData(const std::string& name);

        const std::string& GetName() const;
        const std::vector<std::string>& GetNumber() const;

        void SetName(const std::string& str);
        void SetNumber(const std::vector<std::string>& vec);

        void AddNumber(const char* str);

    private:
        std::string name_;
        std::vector<std::string> number_;
    };

    class PhonebookScanner : public Scanner {
    public:
        PhonebookScanner();

        void GetPhonebook(std::vector<PhonebookData>& book) const;
        unsigned int GetPhonebookNum() const;

    private:
        std::vector<PhonebookData> phonebook_;

        PhonebookScanner(const PhonebookScanner&);
        PhonebookScanner& operator=(const PhonebookScanner&);

        bool DoScan(struct usb_device* dev);
        int Recv(UsbDev& udev, unsigned int offset);

        bool RequestFirst(UsbDev& udev);
        bool RequestRest(UsbDev& udev);
    };

    void GetPhonebookTelStr(std::vector<std::string>& vec, const char* buf, const unsigned int offset);

    void GetPhonebook(std::vector<PhonebookData>& phonebook);
}

/// inline functions.
using namespace KV5500;

inline PhonebookData::PhonebookData(const std::string& name)
    : name_(name)
{}

inline const std::string& PhonebookData::GetName() const
{
    return name_;
}

inline const std::vector<std::string>& PhonebookData::GetNumber() const
{
    return number_;
}

inline void PhonebookData::SetName(const std::string& str)
{
    name_ = str;
}

inline void PhonebookData::SetNumber(const std::vector<std::string>& vec)
{
    number_ = vec;
}

inline void PhonebookData::AddNumber(const char* str)
{
    number_.push_back(str);
}

inline PhonebookScanner::PhonebookScanner()
{}

inline void PhonebookScanner::GetPhonebook(std::vector<PhonebookData>& book) const
{
    book = phonebook_;
}

inline unsigned int PhonebookScanner::GetPhonebookNum() const
{
    return phonebook_.size();
}

#endif
